/***** Zvora - 10key Timer *****/

var isFront = true;
var isFocus;
var resetNextType = true;
var isQuote = false;
var isTimerOn = false;
var isCtdnPaused = false;
var intervalTimer;
var playButtonInterval;
var recallButtonInterval;
var clearButtonInterval;
var dotInterval;
var currentCtdn;
var chime;
var chime1name = "Tea Time";
var chime2name = "Wake Up";
var chime3name = "Noodle Time";
var chime4name = "Short Notice";
var tickSound = new Audio("Sounds/Tick 1mb1.mp3");
tickSound.volume = 0.5;
tickSound.muted = true;
var loopNum = 1;

function loadPrefs() {
	prevCtdn = getFromPref("savedTimer");
	if (prevCtdn && prevCtdn.length > 0) { clearBtn.style.display = "block";}
	else { prevCtdn = 0;}
	alarmMessage = getFromPref("alarmMessage");
	if (!alarmMessage || alarmMessage.length <= 0) { alarmMessage = getLS("Time’s Up!");}
	isMsgStick = getFromPref("isMsgStick");
	if(!isMsgStick) { isMsgStick = false;}
	pbDoPlay = getFromPref("pbDoPlay");
	if(!pbDoPlay){ pbDoPlay = false;}
	pbPlayTime = getFromPref("pbPlayTime");
	if(!pbPlayTime || pbPlayTime.length <=0){ pbPlayTime = "0\'05";}
	pbIsStick = getFromPref("pbIsStick");
	if(!pbIsStick){ pbIsStick = false;}
	isTick = getFromPref("isTick");
	if(isTick){ tickSound.muted = false; }
	else { isTick = false;}
	isLoop = getFromPref("isLoop");
	if(!isLoop){ isLoop = false;}
}

function ctdnFinished() {
	messageAndAlarm("nor");
	animeDot("stop");
	initForNew();
}

function messageAndAlarm(msgType) {
	var appName = "'10key Timer'";
	var notifyTitle;
	var notifyMessage;
	var sticky;
	switch(msgType) {
		case "nor":
			if(chime != null) { chime.play(); }
			if(isMsgStick){ sticky = "yes"; } else { sticky = "no"; }
			notifyTitle = "'" + alarmMessage + "'";
			notifyMessage = "'"+ getLS("The countdown") +" ("+ changeBadChar(prevCtdn) +") "+ getLS("has ended.") +"'";
			break;
		case "pre":
			if(pbChime != null) { pbChime.play(); }
			if(pbIsStick) { sticky = "yes"; } else { sticky = "no"; }
			notifyTitle = "'" + getLS("Last") +" "+ changeBadChar(pbPlayTime) +" "+ getLS("till") +" “"+ alarmMessage + "”'";
			notifyMessage = "'" + getLS("Last") +" "+ changeBadChar(pbPlayTime) +" "+ getLS("till entered time") +" ("+ changeBadChar(prevCtdn) +") "+ getLS("is coming.") + "'";
			if(isLoop) {
				notifyTitle = "'" + getLS("Last") +" "+ changeBadChar(pbPlayTime) +" "+ getLS("till") +" “"+ alarmMessage + "”(#"+ (loopNum + 1) +")'";
				notifyMessage = "'" + getLS("Last") +" "+ changeBadChar(pbPlayTime) +" "+ getLS("till entered time") +" ("+ changeBadChar(prevCtdn) +"): #"+ loopNum +" "+ getLS("passes.") + "'";
			}
		break;
		case "loo":
			if(chime != null) { chime.play(); }
			if(isMsgStick){ sticky = "yes"; } else { sticky = "no"; }
			notifyTitle = "'"+ alarmMessage +" #" + loopNum +" "+ getLS("starts") +"'";
			notifyMessage = "'"+ alarmMessage +" #"+ (loopNum-1) +" ("+ changeBadChar(prevCtdn) +") "+ getLS("has ended.") +"'";
			break;
	}
	var notifyCommand = "/usr/bin/osascript Scripts/growl_alert4.scpt " + appName + " " + notifyTitle + " " + notifyMessage + " " + sticky;
	widget.system(notifyCommand, doneAlert);
}

function doneAlert(){
	//alert("Done Alert.");
}

function changeBadChar(theString) {
	theString = theString.replace(/'/g, "’");
	theString = theString.replace(/"/g, "”");
	return theString;
}
// '
function formatTime(theTime) {
	if(theTime.indexOf("\'") == -1) { theTime = theTime.concat("\'00") };
	var timeArray = theTime.split("\'");
	var secs = Number(timeArray[1])%60;
	if(secs<10) { secs = "0" + secs; }
	var mins = Number(timeArray[0])+Math.floor(timeArray[1]/60);
	return mins + "\'" + secs;
}

function doCountdown() {
	var secs = timeToSec(currentCtdn);
	secs--;
	if (secs > 0) {
		if(pbDoPlay){
			if(secs == timeToSec(pbPlayTime)){
				tickSound.pause();
				messageAndAlarm("pre");
			}
		}
		currentCtdn = formatTime("0\'" + secs);
		if(isTick) { tickSound.play(); }
	}
	else if(isLoop) {
		currentCtdn = prevCtdn;
		loopNum++;
		messageAndAlarm("loo");
	}
	else {
		currentCtdn = "0\'00";
		tickSound.pause();
		ctdnFinished();
	}
	timerField.innerText = currentCtdn;
}

function timeToSec(minsec) {
	var timeArray = minsec.split("\'");
	return Number(timeArray[0]) * 60 + Number(timeArray[1]);
}

function prepareCtdn() {
	var inputTime = timerField.innerText;
	if((inputTime != "0") && (inputTime != "0\'") && (inputTime != "0\'00") && !isTimerOn) {
		changePlayBtn("dn");
		playButtonInterval = setInterval(playButtonUp,300);
		clearBtn.style.display = "block";
		if(!isCtdnPaused) {
			inputTime = formatTime(inputTime);
			timerField.innerText = inputTime;
			setToPref(inputTime, "savedTimer");
			prevCtdn = inputTime;
		} else {
			isCtdnPaused = false;
		}
		currentCtdn = inputTime;
		intervalTimer = setInterval(doCountdown, 1000);
		isTimerOn = true;
		animeDot("start");
		if(isTick) { tickSound.play(); }
	}
}

function ctdnPause() {
	if(isTimerOn){
		changePlayBtn("pausedn");
		playButtonInterval = setInterval(playButtonUp, 300);
		isTimerOn = false;
		isCtdnPaused = true;
		clearInterval(intervalTimer);
		animeDot("pause");
		tickSound.pause();
	}
}

function setSoundPopups(){
	gebi("chime1").innerText = chime1name;
	gebi("chime2").innerText = chime2name;
	gebi("chime3").innerText = chime3name;
	gebi("chime4").innerText = chime4name;
	gebi("pbChime1").innerText = chime1name;
	gebi("pbChime2").innerText = chime2name;
	gebi("pbChime3").innerText = chime3name;
	gebi("pbChime4").innerText = chime4name;
}

function setup() {
	if(window.widget) {
		playBtn = gebi("btPlay");
		recallBtn = gebi("inner_image2");
		loadTheme();
		dot = gebi("playingDot");
		dot.style.display = "none";
		clearBtn = gebi("inner_image5");
		clearBtn.style.display = "none";
		gebi("btUpdate").style.display = "none";
		checkUpdateAvailable();
		setSoundPopups();
		loadPrefs();
		changeOptions("msgStickInput", isMsgStick);
		changeOptions("pbPlayInput", pbDoPlay);
		gebi("pbPlayTimeInput").value = pbPlayTime;
		changeOptions("pbStickInput", pbIsStick);
		changeOptions("tickInput", isTick);
		changeOptions("loopInput", isLoop);
		changePlayBtn("up");
		timerField = gebi("timerField");
		timerField.innerText = prevCtdn;
		initAlarmSound();
		gebi("versionText").innerText = getVersion();
		messageField = gebi("messageField");
		messageField.value = alarmMessage;
		document.addEventListener("keydown",typeFilter,false);
	}
}

function typeFilter(event) {
	if(window.widget && isFocus) {
		if(isFront) {
			switch(event.charCode) {
				case (73):// I
				case (105):// i
					if(event.metaKey) showBack(); break;
				case (63289):// Clear
				case (8):// Delete
					resetAndType(-1); break;
				case (42): // *
				case (82): // R
				case (114): // r
					recallPrevCtdn(); break;
				case (57):
					(resetNextType) ? resetAndType(9) : appendType(9); break;
				case (56):
					(resetNextType) ? resetAndType(8) : appendType(8); break;
				case (55):
					(resetNextType) ? resetAndType(7) : appendType(7); break;
				case (54):
					(resetNextType) ? resetAndType(6) : appendType(6); break;
				case (53):
					(resetNextType) ? resetAndType(5) : appendType(5); break;
				case (52):
					(resetNextType) ? resetAndType(4) : appendType(4); break;
				case (51):
					(resetNextType) ? resetAndType(3) : appendType(3); break;
				case (50):
					(resetNextType) ? resetAndType(2) : appendType(2); break;
				case (49):
					(resetNextType) ? resetAndType(1) : appendType(1); break;
				case (48):
					(resetNextType) ? resetAndType(0) : appendType(0); break;
				case (46): // .
				case (44): // ,
				case (39): // '
					(resetNextType) ? resetAndType("\'") : appendType("\'"); break;
				case (13): // enter or return
					if(event.altKey){
						isLoop = !isLoop;
						changeOptions("loopInput", isLoop);
						changePlayBtn("up");
					}
					(isTimerOn) ? ctdnPause() : prepareCtdn();
					break;
			}
		} else {
			if((event.charCode==73)||(event.charCode==105)&&event.metaKey) { showFront();}
		}
	}
}

function playButtonClick(event) {
	if(event.altKey){
		isLoop = !isLoop;
		changeOptions("loopInput", isLoop);
	}
	(isTimerOn) ? ctdnPause() : prepareCtdn();
}

function clearButtonClick(event) {
	resetAndType(-1);
}

function changePlayBtn(state){
	var aClass;
	switch(state){
		case "up": if(isLoop) { aClass = "loopUp"; } else { aClass = "up"; } break;
		case "dn": if(isLoop) { aClass = "loopDn"; } else { aClass = "dn"; } break;
		case "pauseup": if(isLoop) { aClass = "loopPauseUp"; } else { aClass = "pauseUp"; } break;
		case "pausedn": if(isLoop) { aClass = "loopPauseDn"; } else { aClass = "pauseDn"; } break;
	}
	playBtn.className = aClass;
}


function initForNew() {
	clearInterval(intervalTimer);
	isTimerOn = false;
	isCtdnPaused = false;
	resetNextType = true;
	timerField.innerText = 0;
	changePlayBtn("up");
	animeDot("stop");
}

function resetAndType(num) {
	isQuote = false;
	if(num == -1) {
		tickSound.pause();
		clearBtn.src = "Images/btClear-dn.png";
		clearButtonInterval = setInterval(clearButtonUp,300);
		initForNew();
		num = 0;
	}
	if (!isTimerOn) {
		if(num == 0) {
			resetNextType = true;
			num = 0;
		} else if (num == "\'") {
			num = "0\'";
			resetNextType = false;
			isQuote = true;
		} else {
			resetNextType = false;
		}
		timerField.innerText = num;
	}
}

function appendType(num) {
	if (!isTimerOn && !isCtdnPaused) {
		if (num == "\'") {
			if (isQuote) { num = "";}
			else { isQuote = true;}
		}
		timerField.innerText += num;
	}
}

function recallPrevCtdn() {
	loopNum = 1;
	recallBtn.src = "Images/fRecallBtn-dn.png";
	recallButtonInterval = setInterval(recallButtonUp,300);
	
	if(isTimerOn) {
		initForNew();
		timerField.innerText = prevCtdn;
		prepareCtdn();
	} else if (isCtdnPaused) {
		initForNew();
		resetNextType = true;
		timerField.innerText = prevCtdn;
	} else {
		resetNextType = true;
		timerField.innerText = prevCtdn;
	}
	clearBtn.style.display = "block";
}

function focus() {
	backlight(true);
	isFocus = true;
}

function blur() {
	backlight(false);
	isFocus = false;
}

function backlight(isOn) {
	var backlight = gebi("inner_image1");
	if(isOn) { backlight.src = "Images/backlight-on.png"; }
	else { backlight.src = "Images/backlight-off.png"; }
}

function playButtonUp() {
	clearInterval(playButtonInterval);
	if(isTimerOn) { changePlayBtn("pauseup");}
	else {
		var inputTime = timerField.innerText;
		if((inputTime != "0") && (inputTime != "0\'"))
			changePlayBtn("up");
	}
}

function recallButtonUp() {
	clearInterval(recallButtonInterval);
	recallBtn.src = "Images/fRecallBtn-up.png";
}

function clearButtonUp() {
	clearInterval(clearButtonInterval);
	clearBtn.src = "Images/btClear-up.png";
	clearBtn.style.display = "none";
}


function animeDot(doWhat) {
	switch (doWhat) {
		case ("start"):
			dot.style.display = "block";
			dotInterval = setInterval(dotAnimation, 100);
			break;
		case ("stop"):
			clearInterval(dotInterval);
			dot.style.display = "none";
			dot.style.opacity = 1;
			break;
		case ("pause"):
			clearInterval(dotInterval);
			dot.style.display = "block";
			dot.style.opacity = 1;
	}
}

function dotAnimation() {
	if(dot.style.opacity > 0.4) { dot.style.opacity -= 0.15; }
	else { dot.style.opacity = 1; }
}

function changeAlarmSound(event) {
	var popup = gebi(event.target.id);
	var popupValue = popup.value;
	switch(event.target.id){
		case "AlarmSoundPopup":
			setAlarmSound("nor", popupValue);
			break;
		case "pbAlarmSoundPopup":
			setAlarmSound("pre", popupValue);
	}
}

function initAlarmSound() {
	var savedAlarm = getFromPref("savedAlarm");
	if(!savedAlarm) { savedAlarm = chime1name; }
	setAlarmSound("nor", savedAlarm);
	var pbSavedAlarm = getFromPref("pbSavedAlarm");
	if(!pbSavedAlarm){ pbSavedAlarm = chime4name;}
	setAlarmSound("pre", pbSavedAlarm);
}

function setAlarmSound(norOrPre, popupValue) {
	var popupToChange;
	if(norOrPre == "nor") {
		popupToChange = gebi("AlarmSoundPopup");
		if(popupValue != "«Silence»"){
			var newChime = new Audio("Sounds/"+ popupValue +".mp3");
			chime = newChime;
		} else { chime = null;}
		setToPref(popupValue, "savedAlarm");
		if(popupToChange.value != popupValue) { popupToChange.value = popupValue;}
	} else {
		popupToChange = gebi("pbAlarmSoundPopup");
		if(popupValue != "«Silence»"){
			var pbNewChime = new Audio("Sounds/"+ popupValue +".mp3");
			pbChime = pbNewChime;
		} else { pbChime = null;}
		setToPref(popupValue, "pbSavedAlarm");
		if(popupToChange.value != popupValue) { popupToChange.value = popupValue;}
	}
}

function setAlarmMessage() {
    var newAlarmMessage = messageField.value;
	if(newAlarmMessage && newAlarmMessage.length > 0) {
		alarmMessage = changeBadChar(newAlarmMessage);
		messageField.value = alarmMessage;
		setToPref(alarmMessage, "alarmMessage");
	}
}

function loadTheme() {
	var lastTheme = getFromPref("lastTheme");
	if(!lastTheme) {
		lastTheme = 2;
		setToPref(lastTheme, "lastTheme");
	}
	setTheme(lastTheme);
	gebi("btTheme" + lastTheme).src = "Images/btTheme" + themeFromNumber(lastTheme) + "_dn.png";
}

function setTheme(themeNumber) {
	gebi("baseImg").src = "Images/base" + themeFromNumber(themeNumber) + ".png";
	setToPref(themeNumber, "lastTheme");
}

function themeFromNumber(themeNumber) {
	var theTheme = "";
	switch(themeNumber) {
		case 1: theTheme = "Black"; break;
		case 2: theTheme = "Silver"; break;
		case 3: theTheme = "Pink"; break;
		case 4: theTheme = "Green"; break;
		case 5: theTheme = "Blue"; break;
		defalut: theTheme = "Silver";
	}
	return theTheme;	
}
